@@ -1,3 +1,6 @@
+import argparse
+from typing import Any, List
+
 from django.conf import settings
 from django.core.cache import InvalidCacheBackendError, caches
 from django.core.management import BaseCommand, CommandError
@@ -16,7 +19,7 @@ class Command(BaseCommand):
     """
     )
 
-    def add_arguments(self, parser):
+    def add_arguments(self, parser: argparse.ArgumentParser) -> None:
         parser.add_argument(
             "aliases",
             metavar="aliases",
@@ -24,27 +27,23 @@ class Command(BaseCommand):
             help="Specify the cache alias(es) to cull.",
         )
 
-    def handle(self, *args, **options):
-        verbosity = options.get("verbosity")
-
-        aliases = set(options["aliases"])
-
+    def handle(
+        self, *args: Any, verbosity: int, aliases: List[str], **options: Any
+    ) -> None:
         if not aliases:
-            aliases = settings.CACHES
+            aliases = list(settings.CACHES)
 
         for alias in aliases:
             try:
                 cache = caches[alias]
             except InvalidCacheBackendError:
-                raise CommandError("Cache '{}' does not exist".format(alias))
+                raise CommandError(f"Cache '{alias}' does not exist")
 
             if not isinstance(cache, MySQLCache):  # pragma: no cover
                 continue
 
             if verbosity >= 1:
-                self.stdout.write(
-                    "Deleting from cache '{}'... ".format(alias), ending=""
-                )
+                self.stdout.write(f"Deleting from cache '{alias}'... ", ending="")
             num_deleted = cache.cull()
             if verbosity >= 1:
-                self.stdout.write("{} entries deleted.".format(num_deleted))
+                self.stdout.write(f"{num_deleted} entries deleted.")
