@@ -3,11 +3,14 @@ import json
 from typing import Any, Dict, List, Optional, Tuple, Type, Union
 
 from django.db import DEFAULT_DB_ALIAS, connections
+from django.db.backends.base.base import BaseDatabaseWrapper
 from django.db.models import CharField, Expression
 from django.db.models import Field as DjangoField
 from django.db.models import Func, IntegerField, TextField, Value
+from django.db.models.sql.compiler import SQLCompiler
 
 from django_mysql.compat import HAVE_JSONFIELD, JSONField
+from django_mysql.utils import connection_is_mariadb
 
 ExpressionArgument = Union[
     Expression,
@@ -17,12 +20,11 @@ ExpressionArgument = Union[
 
 class SingleArgFunc(Func):
 
-    output_field_class: Optional[Type[DjangoField]] = None
+    output_field_class: Type[DjangoField]
 
     def __init__(self, expression: ExpressionArgument) -> None:
         super().__init__(expression)
-        if self.output_field_class is not None:
-            self.output_field = self.output_field_class()
+        self.output_field = self.output_field_class()
 
 
 # Control Flow Functions
@@ -204,7 +206,7 @@ class LastInsertId(Func):
 
 # JSON Functions
 
-if HAVE_JSONFIELD:
+if HAVE_JSONFIELD:  # pragma: no branch
 
     class JSONExtract(Func):
         function = "JSON_EXTRACT"
@@ -270,15 +272,27 @@ if HAVE_JSONFIELD:
 
             super().__init__(*exprs, output_field=output_field)
 
-    class JSONValue(Func):
-        function = "CAST"
-        template = "%(function)s(%(expressions)s AS JSON)"
-
+    # When only Django 3.1+ is supported, JSONValue can be replaced with
+    # Cast(..., output_field=JSONField())
+    class JSONValue(Expression):
         def __init__(
-            self, expression: Union[None, int, float, str, List[Any], Dict[str, Any]]
+            self, data: Union[None, int, float, str, List[Any], Dict[str, Any]]
         ) -> None:
-            json_string = json.dumps(expression, allow_nan=False)
-            super().__init__(Value(json_string))
+            self._data = data
+
+        def as_sql(
+            self,
+            compiler: SQLCompiler,
+            connection: BaseDatabaseWrapper,
+        ) -> Tuple[str, Tuple[Any, ...]]:
+            if connection.vendor != "mysql":  # pragma: no cover
+                raise AssertionError("JSONValue only supports MySQL/MariaDB")
+            json_string = json.dumps(self._data, allow_nan=False)
+            if connection_is_mariadb(connection):
+                # MariaDB doesn't support explicit cast to JSON.
+                return "JSON_EXTRACT(%s, '$')", (json_string,)
+            else:
+                return "CAST(%s AS JSON)", (json_string,)
 
     class BaseJSONModifyFunc(Func):
         def __init__(
